/*
 * Warning: do not edit this file, it has possibly been generated by configure
 *
 * TELESCOPE=carma
 */
#define         MIRTEL          "carma"
#define		MAXDIM		65536
#define		MAXDIM2		8192
#define         MAXIANT         2048
#define		MAXANT		2048
#define		MAXANT2		28
#if 0
#define         MAXBASE         ((MAXANT * (MAXANT + 1)) / 2) */
#else
#define		MAXBASE		524800
#endif
#define		MAXBASE2	500
#define		MAXCHAN		32768
#define		MAXNAX		7
#define		MAXWIN		16
#define		MAXBUF		4194304

