#ifndef _GRID_H_
#define _GRID_H_

#include <stdlib.h>
#include <math.h>

int grid1D_r(float *, long, float *, float *, long, long);
int grid1D_c(float *, long, float *, float *, long, long);
int grid2D_c(float *, long, long, float *, float *, float *, long, long);
int degrid2D_c(float *, long, long, float *, float *, float *, long, long);

#endif
